@@ -48,7 +48,7 @@ static int cicero_speaking = 0;
 static pthread_t cicero_speaking_thread;
 static sem_t cicero_semaphore;
 
-static char **cicero_message;
+static char *cicero_message;
 static SPDMessageType cicero_message_type;
 
 static int cicero_position = 0;
@@ -148,6 +148,7 @@ int module_init(char **status_info)
 	DBG("Module init\n");
 
 	if (access(CiceroExecutable, X_OK) != 0) {
+		*status_info = g_strdup_printf("can not reach executable %s", CiceroExecutable);
 		DBG("ERROR: can not reach executable %s", CiceroExecutable);
 		return -1;
 	}
@@ -156,6 +157,7 @@ int module_init(char **status_info)
 
 	DBG("call the pipe system call\n");
 	if (pipe(fd1) < 0 || pipe(fd2) < 0) {
+		*status_info = g_strdup("Could not create pipe");
 		DBG("Error pipe()\n");
 		return -1;
 	}
@@ -171,6 +173,7 @@ int module_init(char **status_info)
 	}
 	switch (fork()) {
 	case -1:{
+			*status_info = g_strdup("Could not fork");
 			DBG("Error fork()\n");
 			return -1;
 		}
@@ -208,15 +211,14 @@ int module_init(char **status_info)
 		}
 	}
 
-	cicero_message = g_malloc(sizeof(char *));
-	*cicero_message = NULL;
+	cicero_message = NULL;
 
 	sem_init(&cicero_semaphore, 0, 0);
 
 	DBG("Cicero: creating new thread for cicero_tracking\n");
 	cicero_speaking = 0;
 	ret =
-	    pthread_create(&cicero_speaking_thread, NULL, _cicero_speak, NULL);
+	    spd_pthread_create(&cicero_speaking_thread, NULL, _cicero_speak, NULL);
 	if (ret != 0) {
 		DBG("Cicero: thread failed\n");
 		*status_info =
@@ -251,11 +253,12 @@ int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
 
 	DBG("Requested data: |%s|\n", data);
 
-	if (*cicero_message != NULL) {
-		g_free(*cicero_message);
-		*cicero_message = NULL;
+	if (cicero_message != NULL) {
+		g_free(cicero_message);
+		cicero_message = NULL;
 	}
-	*cicero_message = module_strip_ssml(data);
+	cicero_message = module_strip_ssml(data);
+	/* TODO: use a generic engine for SPELL, CHAR, KEY */
 	cicero_message_type = SPD_MSGTYPE_TEXT;
 
 	/* Setting voice */
@@ -326,11 +329,12 @@ void *_cicero_speak(void *nothing)
 	struct pollfd ufds = { fd1[0], POLLIN | POLLPRI, 0 };
 
 	DBG("cicero: speaking thread starting.......\n");
+	/* Make interruptible */
 	set_speaking_thread_parameters();
 	while (1) {
 		sem_wait(&cicero_semaphore);
 		DBG("Semaphore on\n");
-		len = strlen(*cicero_message);
+		len = strlen(cicero_message);
 		cicero_stop = 0;
 		cicero_speaking = 1;
 		cicero_position = 0;
@@ -351,9 +355,9 @@ void *_cicero_speak(void *nothing)
 				break;
 			}
 			DBG("Call get_parts: pos=%d, msg=\"%s\" \n", pos,
-			    *cicero_message);
+			    cicero_message);
 			bytes =
-			    module_get_message_part(*cicero_message, buf, &pos,
+			    module_get_message_part(cicero_message, buf, &pos,
 						    CiceroMaxChunkLength,
 						    ".;?!");
 			DBG("Returned %d bytes from get_part\n", bytes);
