@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: speechd.c,v 1.81 2008-07-10 15:36:49 hanke Exp $
  */
@@ -32,6 +30,10 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 
+#ifdef HAVE_SYS_FILIO_H
+#include <sys/filio.h>	/* Needed for FIONREAD on Solaris */
+#endif
+
 #include "speechd.h"
 
 /* Declare dotconf functions and data structures*/
@@ -42,6 +44,7 @@
 #include "alloc.h"
 #include "sem_functions.h"
 #include "speaking.h"
+#include "speak_queue.h"
 #include "set.h"
 #include "options.h"
 #include "server.h"
@@ -58,12 +61,62 @@ void destroy_pid_file();
 /* Server socket file descriptor */
 int server_socket;
 
-void speechd_load_configuration(int sig);
+GMainLoop *main_loop = NULL;
+gint server_timeout_source = -1;
 
-#ifdef __SUNPRO_C
-/* Added by Willie Walker - daemon is a gcc-ism
+int client_count = 0;
+
+struct SpeechdOptions SpeechdOptions;
+struct SpeechdStatus SpeechdStatus;
+
+pthread_t speak_thread;
+pthread_mutex_t logging_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t element_free_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t output_layer_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t socket_com_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+GHashTable *fd_settings;
+GHashTable *language_default_modules;
+GHashTable *fd_uid;
+
+TSpeechDQueue *MessageQueue;
+GList *MessagePausedList;
+
+GList *client_specific_settings;
+
+GList *last_p5_block;
+
+TFDSetElement GlobalFDSet;
+
+int speaking_pipe[2];
+
+GHashTable *speechd_sockets_status;
+
+FILE *logfile;
+FILE *custom_logfile;
+char *custom_log_kind;
+FILE *debug_logfile;
+
+TSpeechDMode spd_mode;
+
+static gboolean speechd_client_terminate(gpointer key, gpointer value, gpointer user);
+static gboolean speechd_reload_dead_modules(gpointer user_data);
+static gboolean speechd_load_configuration(gpointer user_data);
+static gboolean speechd_quit(gpointer user_data);
+
+static gboolean server_process_incoming (gint          fd,
+				  GIOCondition  condition,
+				  gpointer      data);
+
+static gboolean client_process_incoming (gint          fd,
+				  GIOCondition  condition,
+				  gpointer      data);
+
+void check_client_count(void);
+
+#ifndef HAVE_DAEMON
+/* Added by Willie Walker - daemon is a common, but not universal, extension.
  */
-#include <sys/filio.h>
 static int daemon(int nochdir, int noclose)
 {
 	int fd, i;
@@ -97,23 +150,7 @@ static int daemon(int nochdir, int noclose)
 	}
 	return 0;
 }
-#endif /* __SUNPRO_C */
-
-char *spd_get_path(char *filename, char *startdir)
-{
-	char *ret;
-	if (filename == NULL)
-		return NULL;
-	if (filename[0] != '/') {
-		if (startdir == NULL)
-			ret = g_strdup(filename);
-		else
-			ret = g_strdup_printf("%s/%s", startdir, filename);
-	} else {
-		ret = g_strdup(filename);
-	}
-	return ret;
-}
+#endif /* HAVE_DAEMON */
 
 /* --- DEBUGGING --- */
 
@@ -126,7 +163,7 @@ void fatal_error(void)
 
 /* Logging messages, level of verbosity is defined between 1 and 5,
  * see documentation */
-void MSG2(int level, char *kind, char *format, ...)
+void MSG2(int level, const char *kind, const char *format, ...)
 {
 	int std_log = level <= SpeechdOptions.log_level;
 	int custom_log = (kind != NULL && custom_log_kind != NULL &&
@@ -211,7 +248,7 @@ void MSG2(int level, char *kind, char *format, ...)
    documentation */
 /* TODO: Define this in terms of MSG somehow. I don't
    know how to pass '...' arguments to another C function */
-void MSG(int level, char *format, ...)
+void MSG(int level, const char *format, ...)
 {
 
 	if ((level <= SpeechdOptions.log_level)
@@ -349,7 +386,6 @@ int speechd_connection_new(int server_socket)
 	}
 
 	/* We add the associated client_socket to the descriptor set. */
-	FD_SET(client_socket, &readfds);
 	if (client_socket > SpeechdStatus.max_fd)
 		SpeechdStatus.max_fd = client_socket;
 	MSG(4, "Adding client on fd %d", client_socket);
@@ -363,7 +399,6 @@ int speechd_connection_new(int server_socket)
 		    "Error: Failed to create a record in fd_settings for the new client");
 		if (SpeechdStatus.max_fd == client_socket)
 			SpeechdStatus.max_fd--;
-		FD_CLR(client_socket, &readfds);
 		return -1;
 	}
 	new_fd_set->fd = client_socket;
@@ -378,8 +413,15 @@ int speechd_connection_new(int server_socket)
 	g_hash_table_insert(fd_settings, p_client_uid, new_fd_set);
 	g_hash_table_insert(fd_uid, p_client_socket, p_client_uid2);
 
+	new_fd_set->fd_source = g_unix_fd_add(client_socket, G_IO_IN, client_process_incoming, NULL);
+
 	MSG(4, "Data structures for client on fd %d created", client_socket);
+
+	client_count++;
+	check_client_count();
+
 	return 0;
+
 }
 
 int speechd_connection_destroy(int fd)
@@ -394,6 +436,7 @@ int speechd_connection_destroy(int fd)
 	if (fdset_element != NULL) {
 		fdset_element->fd = -1;
 		fdset_element->active = 0;
+		g_source_remove(fdset_element->fd_source);
 		/* The fdset_element will be freed and removed from the
 		   hash table as soon as the client no longer has any
 		   message in the queues, check out the speak() function */
@@ -413,16 +456,18 @@ int speechd_connection_destroy(int fd)
 		if (SPEECHD_DEBUG)
 			DIE("Can't close file descriptor associated to this client");
 
-	FD_CLR(fd, &readfds);
 	if (fd == SpeechdStatus.max_fd)
 		SpeechdStatus.max_fd--;
 
 	MSG(4, "Connection closed");
 
+	client_count--;
+	check_client_count();
+
 	return 0;
 }
 
-gboolean speechd_client_terminate(gpointer key, gpointer value, gpointer user)
+static gboolean speechd_client_terminate(gpointer key, gpointer value, gpointer user)
 {
 	TFDSetElement *set;
 
@@ -504,13 +549,14 @@ void speechd_module_nodebug(gpointer data, gpointer user_data)
 	return;
 }
 
-void speechd_reload_dead_modules(int sig)
+static gboolean speechd_reload_dead_modules(gpointer user_data)
 {
 	/* Reload dead modules */
 	g_list_foreach(output_modules, speechd_modules_reload, NULL);
 
 	/* Make sure there aren't any more child processes left */
 	while (waitpid(-1, NULL, WNOHANG) > 0) ;
+	return TRUE;
 }
 
 void speechd_modules_debug(void)
@@ -538,7 +584,11 @@ void speechd_options_init(void)
 	SpeechdOptions.localhost_access_only_set = 0;
 	SpeechdOptions.pid_file = NULL;
 	SpeechdOptions.conf_file = NULL;
+	SpeechdOptions.conf_dir = NULL;
+	SpeechdOptions.user_conf_dir = NULL;
 	SpeechdOptions.runtime_speechd_dir = NULL;
+	SpeechdOptions.module_dir = MODULEBINDIR;
+	SpeechdOptions.user_module_dir = NULL;
 	SpeechdOptions.log_dir = NULL;
 	SpeechdOptions.log_dir_set = 0;
 	SpeechdOptions.debug = 0;
@@ -549,8 +599,6 @@ void speechd_options_init(void)
 
 void speechd_init()
 {
-	int ret;
-
 	SpeechdStatus.max_uid = 0;
 	SpeechdStatus.max_gid = 0;
 
@@ -563,7 +611,7 @@ void speechd_init()
 	/* Initialize Speech Dispatcher priority queue */
 	MessageQueue = g_malloc0(sizeof(TSpeechDQueue));
 	if (MessageQueue == NULL)
-		FATAL("Couldn't alocate memmory for MessageQueue.");
+		FATAL("Couldn't allocate memory for MessageQueue.");
 
 	/* Initialize lists */
 	MessagePausedList = NULL;
@@ -599,19 +647,6 @@ void speechd_init()
 	/* Initialize list of different client specific settings entries */
 	client_specific_settings = NULL;
 
-	/* Initialize mutexes, semaphores and synchronization */
-	ret = pthread_mutex_init(&element_free_mutex, NULL);
-	if (ret != 0)
-		DIE("Mutex initialization failed");
-
-	ret = pthread_mutex_init(&output_layer_mutex, NULL);
-	if (ret != 0)
-		DIE("Mutex initialization failed");
-
-	ret = pthread_mutex_init(&socket_com_mutex, NULL);
-	if (ret != 0)
-		DIE("Mutex initialization failed");
-
 	if (SpeechdOptions.log_dir == NULL) {
 		SpeechdOptions.log_dir =
 		    g_strdup_printf("%s/log/",
@@ -628,7 +663,7 @@ void speechd_init()
 	/* Load configuration from the config file */
 	MSG(4, "Reading Speech Dispatcher configuration from %s",
 	    SpeechdOptions.conf_file);
-	speechd_load_configuration(0);
+	speechd_load_configuration(NULL);
 
 	logging_init();
 
@@ -644,7 +679,63 @@ void speechd_init()
 	last_p5_block = NULL;
 }
 
-void speechd_load_configuration(int sig)
+static gint modules_compare (gconstpointer a, gconstpointer b)
+{
+	const char **params_a = (const char **) a;
+	const char *name_a = params_a[0];
+	const char **params_b = (const char **) b;
+	const char *name_b = params_b[0];
+	unsigned index_a;
+	unsigned index_b;
+
+	/* This gives the prioritization order of modules, to automatically select the best quality */
+	static const char *modules_order[] = {
+		"voxin",
+		"ivona",
+		"pico",
+		"pico-generic",
+		"rhvoice",
+		"cicero",
+		"kali",
+		"ibmtts",
+		"festival",
+		"flite",
+		"espeak-ng-mbrola",
+		"espeak-ng-mbrola-generic",
+		"espeak-ng",
+		"espeak-mbrola-generic",
+		"espeak",
+		"espeak-generic",
+	};
+	static const int index_max = sizeof(modules_order) / sizeof(*modules_order);
+
+	for (index_a = 0; index_a < index_max; index_a++) {
+		if (!strcmp(modules_order[index_a], name_a))
+			break;
+	}
+
+	for (index_b = 0; index_b < index_max; index_b++) {
+		if (!strcmp(modules_order[index_b], name_b))
+			break;
+	}
+
+	if (index_a != index_max && index_b == index_max)
+		/* We prefer a synth in the list */
+		return -1;
+
+	if (index_a == index_max && index_b != index_max)
+		/* We prefer a synth in the list */
+		return 1;
+
+	if (index_a != index_max && index_b != index_max)
+		/* We follow the ordering */
+		return index_a - index_b;
+
+	/* We fall back to alphabetical ordering */
+	return strcmp(name_a, name_b);
+}
+
+static gboolean speechd_load_configuration(gpointer user_data)
 {
 	configfile_t *configfile = NULL;
 	GList *detected_modules = NULL;
@@ -673,6 +764,7 @@ void speechd_load_configuration(int sig)
 	    dotconf_create(SpeechdOptions.conf_file, spd_options, 0,
 			   CASE_INSENSITIVE);
 	if (configfile) {
+		free(configfile->includepath);
 		configfile->includepath = g_strdup(SpeechdOptions.conf_dir);
 		MSG(5, "Config file include path is: %s",
 		    configfile->includepath);
@@ -684,14 +776,30 @@ void speechd_load_configuration(int sig)
 
 		/* We need to load modules here, since this is called both by speechd_init
 		 * and to handle SIGHUP. */
-		if (module_number_of_requested_modules() < 2) {
-			detected_modules = detect_output_modules(MODULEBINDIR);
+		if (module_number_of_requested_modules() < 1) {
+			detected_modules = detect_output_modules(NULL,
+								 SpeechdOptions.user_module_dir,
+								 SpeechdOptions.user_conf_dir,
+								 SpeechdOptions.conf_dir);
+			detected_modules = detect_output_modules(detected_modules,
+								 SpeechdOptions.module_dir,
+								 SpeechdOptions.user_conf_dir,
+								 SpeechdOptions.conf_dir);
+			detected_modules = detect_output_modules(detected_modules,
+								 OLDMODULEBINDIR,
+								 SpeechdOptions.user_conf_dir,
+								 SpeechdOptions.conf_dir);
+
+			detected_modules = g_list_sort(detected_modules, modules_compare);
+
 			while (detected_modules != NULL) {
 				char **parameters = detected_modules->data;
 				module_add_load_request(parameters[0],
 							parameters[1],
 							parameters[2],
-							parameters[3]);
+							parameters[3],
+							parameters[4],
+							parameters[5]);
 				g_free(detected_modules->data);
 				detected_modules->data = NULL;
 				detected_modules =
@@ -706,47 +814,14 @@ void speechd_load_configuration(int sig)
 	}
 
 	free_config_options(spd_options, &spd_num_options);
+
+	return TRUE;
 }
 
-void speechd_quit(int sig)
+static gboolean speechd_quit(gpointer user_data)
 {
-	int ret;
-
-	MSG(1, "Terminating...");
-
-	MSG(2, "Closing open connections...");
-	/* We will browse through all the connections and close them. */
-	g_hash_table_foreach_remove(fd_settings, speechd_client_terminate,
-				    NULL);
-	g_hash_table_destroy(fd_settings);
-
-	MSG(4, "Closing speak() thread...");
-	ret = pthread_cancel(speak_thread);
-	if (ret != 0)
-		FATAL("Speak thread failed to cancel!\n");
-
-	ret = pthread_join(speak_thread, NULL);
-	if (ret != 0)
-		FATAL("Speak thread failed to join!\n");
-
-	MSG(2, "Closing open output modules...");
-	/*  Call the close() function of each registered output module. */
-	g_list_foreach(output_modules, speechd_modules_terminate, NULL);
-	g_list_free(output_modules);
-
-	MSG(2, "Closing server connection...");
-	if (close(server_socket) == -1)
-		MSG(2, "close() failed: %s", strerror(errno));
-	FD_CLR(server_socket, &readfds);
-
-	MSG(4, "Removing pid file");
-	destroy_pid_file();
-
-	fflush(NULL);
-
-	MSG(2, "Speech Dispatcher terminated correctly");
-
-	exit(0);
+	g_main_loop_quit(main_loop);
+	return FALSE;
 }
 
 /* --- PID FILES --- */
@@ -904,7 +979,7 @@ int make_inet_socket(const int port)
 
 	server_address.sin_port = htons(port);
 
-	MSG(4, "Openning inet socket connection");
+	MSG(4, "Opening inet socket connection");
 	if (bind(server_socket, (struct sockaddr *)&server_address,
 		 sizeof(server_address)) == -1) {
 		MSG(-1, "bind() failed: %s", strerror(errno));
@@ -920,25 +995,76 @@ int make_inet_socket(const int port)
 	return server_socket;
 }
 
+gboolean server_process_incoming (gint          fd,
+				  GIOCondition  condition,
+				  gpointer      data)
+{
+	int ret;
+
+	ret = speechd_connection_new(fd);
+	if (ret != 0) {
+		MSG(2, "Error: Failed to add new client!");
+		if (SPEECHD_DEBUG) {
+			FATAL("Failed to add new client");
+		}
+	}
+
+	return TRUE;
+}
+
+gboolean client_process_incoming (gint          fd,
+				  GIOCondition  condition,
+				  gpointer      data)
+{
+	int ret;
+	int nread;
+
+	ioctl(fd, FIONREAD, &nread);
+
+	if (nread == 0) {
+		/* client has gone */
+		ret = speechd_connection_destroy(fd);
+		if (ret != 0) {
+			MSG(2, "Error: Failed to close the client!");
+		}
+		return FALSE;
+	}
+
+	/* client sends some commands or data */
+	if (serve(fd) == -1) {
+		MSG(2, "Error: Failed to serve client on fd %d!", fd);
+	}
+
+	return TRUE;
+}
+
+void check_client_count(void)
+{
+	if (client_count <= 0
+	    && SpeechdOptions.server_timeout > 0) {
+		MSG(4, "Currently no clients connected, enabling shutdown timer.");
+		server_timeout_source = g_timeout_add_seconds(
+		                        SpeechdOptions.server_timeout,
+		                        speechd_quit, NULL);
+	} else {
+		if (server_timeout_source >= 0) {
+			MSG(4, "Clients connected, disabling shutdown timer.");
+			g_source_remove(server_timeout_source);
+			server_timeout_source = -1;
+		}
+	}
+}
+
 /* --- MAIN --- */
 
 int main(int argc, char *argv[])
 {
-	fd_set testfds;
-	int fd;
 	int ret;
 	/* Autospawn helper variables */
 	char *spawn_communication_method = NULL;
 	int spawn_port = 0;
 	char *spawn_socket_path = NULL;
 
-	/* Initialize threading and thread safety in Glib
-	 * Deprecated in 2.32
-	 */
-#ifndef GLIB_VERSION_2_32
-	g_thread_init(NULL);
-#endif
-
 	/* Strip all permisions for 'others' from the files created */
 	umask(007);
 
@@ -984,10 +1110,11 @@ int main(int argc, char *argv[])
 	{
 		const char *user_runtime_dir;
 		const char *user_config_dir;
-		char *test_speechd_conf_file = NULL;
+		const char *user_data_dir;
 
 		user_runtime_dir = g_get_user_runtime_dir();
 		user_config_dir = g_get_user_config_dir();
+		user_data_dir = g_get_user_data_dir();
 
 		/* Setup a speechd-dispatcher directory or create a new one */
 		SpeechdOptions.runtime_speechd_dir =
@@ -1003,32 +1130,41 @@ int main(int argc, char *argv[])
 			SpeechdOptions.pid_file =
 			    g_strdup_printf("%s/pid/speech-dispatcher.pid",
 					    SpeechdOptions.runtime_speechd_dir);
-			g_mkdir(g_path_get_dirname(SpeechdOptions.pid_file),
-				S_IRWXU);
+			gchar *dirname = g_path_get_dirname(SpeechdOptions.pid_file);
+			g_mkdir(dirname, S_IRWXU);
+			g_free(dirname);
 		}
 		/* Config file */
-		if (SpeechdOptions.conf_dir == NULL) {
+		if (SpeechdOptions.conf_dir) {
+			SpeechdOptions.conf_file =
+			    g_strdup_printf("%s/speechd.conf", SpeechdOptions.conf_dir);
+		} else {
 			/* If no conf_dir was specified on command line, try default local config dir */
-			SpeechdOptions.conf_dir =
+			if (strcmp(SYS_CONF, ""))
+				SpeechdOptions.conf_dir =
+				    g_strdup(SYS_CONF);
+			else
+				SpeechdOptions.conf_dir =
+				    g_strdup("/etc/speech-dispatcher/");
+			SpeechdOptions.user_conf_dir =
 			    g_build_filename(user_config_dir,
 					     "speech-dispatcher", NULL);
-			test_speechd_conf_file =
-			    g_build_filename(SpeechdOptions.conf_dir,
-					     "speechd.conf", NULL);
+
+			MSG(4, "Looking for configuration in %s", SpeechdOptions.user_conf_dir);
+			SpeechdOptions.conf_file =
+			    g_strdup_printf("%s/speechd.conf", SpeechdOptions.user_conf_dir);
 			if (!g_file_test
-			    (test_speechd_conf_file, G_FILE_TEST_IS_REGULAR)) {
+			    (SpeechdOptions.conf_file, G_FILE_TEST_IS_REGULAR)) {
 				/* If the local configuration file doesn't exist, read the global configuration */
-				if (strcmp(SYS_CONF, ""))
-					SpeechdOptions.conf_dir =
-					    g_strdup(SYS_CONF);
-				else
-					SpeechdOptions.conf_dir =
-					    g_strdup("/etc/speech-dispatcher/");
+				g_free(SpeechdOptions.conf_file);
+				SpeechdOptions.conf_file =
+				    g_strdup_printf("%s/speechd.conf", SpeechdOptions.conf_dir);
 			}
-			g_free(test_speechd_conf_file);
 		}
-		SpeechdOptions.conf_file =
-		    g_strdup_printf("%s/speechd.conf", SpeechdOptions.conf_dir);
+
+		SpeechdOptions.user_module_dir =
+			g_strdup_printf("%s/../libexec/speech-dispatcher-modules", user_data_dir);
+		MSG(4, "User module dir is %s", SpeechdOptions.user_module_dir);
 	}
 
 	/* Check for PID file or create a new one or exit if Speech Dispatcher
@@ -1048,7 +1184,7 @@ int main(int argc, char *argv[])
 		    g_file_get_contents(SpeechdOptions.conf_file,
 					&config_contents, NULL, NULL);
 		if (err == FALSE) {
-			MSG(-1, "Error openning %s", SpeechdOptions.conf_file);
+			MSG(-1, "Error opening %s", SpeechdOptions.conf_file);
 			FATAL("Can't open conf file");
 		}
 		regexp =
@@ -1065,20 +1201,6 @@ int main(int argc, char *argv[])
 		MSG(2, "Starting Speech Dispatcher due to auto-spawn");
 	}
 
-	/* Initialize logging mutex to workaround ctime threading bug */
-	/* Must be done no later than here */
-	ret = pthread_mutex_init(&logging_mutex, NULL);
-	if (ret != 0) {
-		fprintf(stderr, "Mutex initialization failed");
-		exit(1);
-	}
-
-	/* Register signals */
-	(void)signal(SIGINT, speechd_quit);
-	(void)signal(SIGHUP, speechd_load_configuration);
-	(void)signal(SIGPIPE, SIG_IGN);
-	(void)signal(SIGUSR1, speechd_reload_dead_modules);
-
 	speechd_init();
 
 	/* Handle socket_path 'default' */
@@ -1192,70 +1314,76 @@ int main(int argc, char *argv[])
 			return -1;
 	}
 
+	/* Set up the main loop and register signals */
+        main_loop = g_main_loop_new(g_main_context_default(), FALSE);
+	g_unix_signal_add(SIGINT, speechd_quit, NULL);
+	g_unix_signal_add(SIGTERM, speechd_quit, NULL);
+	g_unix_signal_add(SIGHUP, speechd_load_configuration, NULL);
+	g_unix_signal_add(SIGUSR1, speechd_reload_dead_modules, NULL);
+	(void)signal(SIGPIPE, SIG_IGN);
+
 	MSG(4, "Creating new thread for speak()");
 	ret = pthread_create(&speak_thread, NULL, speak, NULL);
 	if (ret != 0)
 		FATAL("Speak thread failed!\n");
 
-	FD_ZERO(&readfds);
-	FD_SET(server_socket, &readfds);
+	char *status;
+	ret = module_speak_queue_init(SpeechdOptions.max_queue_size, &status);
+	if (ret != 0)
+		FATAL("Speak queue thread failed: %s!\n", status);
+
 	SpeechdStatus.max_fd = server_socket;
 
+	g_unix_fd_add(server_socket, G_IO_IN,
+		      server_process_incoming, NULL);
+
 	/* Now wait for clients and requests. */
 	MSG(1, "Speech Dispatcher started and waiting for clients ...");
-	while (1) {
-		testfds = readfds;
 
-		if (select
-		    (FD_SETSIZE, &testfds, (fd_set *) 0, (fd_set *) 0,
-		     NULL) >= 1) {
-			/* Once we know we've got activity,
-			 * we find which descriptor it's on by checking each in turn using FD_ISSET.
-			 TODO: This is a hack. We should not be browsing all possible socket numbers
-			 but only those which are connected.
-			 */
-			for (fd = 0;
-			     fd <= SpeechdStatus.max_fd && fd < FD_SETSIZE;
-			     fd++) {
-				if (FD_ISSET(fd, &testfds)) {
-					MSG(4, "Activity on fd %d ...", fd);
+	check_client_count();
 
-					if (fd == server_socket) {
-						/* server activity (new client) */
-						ret =
-						    speechd_connection_new
-						    (server_socket);
-						if (ret != 0) {
-							MSG(2,
-							    "Error: Failed to add new client!");
-							if (SPEECHD_DEBUG)
-								FATAL
-								    ("Failed to add new client");
-						}
-					} else {
-						/* client activity */
-						int nread;
-						ioctl(fd, FIONREAD, &nread);
+	g_main_loop_run(main_loop);
 
-						if (nread == 0) {
-							/* client has gone */
-							speechd_connection_destroy
-							    (fd);
-							if (ret != 0)
-								MSG(2,
-								    "Error: Failed to close the client!");
-						} else {
-							/* client sends some commands or data */
-							if (serve(fd) == -1)
-								MSG(2,
-								    "Error: Failed to serve client on fd %d!",
-								    fd);
-						}
-					}
-				}
-			}
-		}
-	}
+	MSG(1, "Terminating...");
+
+	MSG(2, "Closing open connections...");
+	/* We will browse through all the connections and close them. */
+	g_hash_table_foreach_remove(fd_settings, speechd_client_terminate,
+				    NULL);
+	g_hash_table_destroy(fd_settings);
+
+	MSG(4, "Closing speak() thread...");
+	ret = pthread_cancel(speak_thread);
+	if (ret != 0)
+		FATAL("Speak thread failed to cancel!\n");
+
+	ret = pthread_join(speak_thread, NULL);
+	if (ret != 0)
+		FATAL("Speak thread failed to join!\n");
+
+	MSG(4, "Closing play() thread...");
+	module_speak_queue_terminate();
+
+	MSG(2, "Closing open output modules...");
+	/*  Call the close() function of each registered output module. */
+	g_list_foreach(output_modules, speechd_modules_terminate, NULL);
+	g_list_free(output_modules);
+
+	MSG(2, "Closing server connection...");
+	if (close(server_socket) == -1)
+		MSG(2, "close() failed: %s", strerror(errno));
+
+	MSG(4, "Removing pid file");
+	destroy_pid_file();
+
+	fflush(NULL);
+
+	g_main_loop_unref(main_loop);
+	main_loop = NULL;
+
+	MSG(2, "Speech Dispatcher terminated correctly");
+
+	exit(0);
 }
 
 void check_locked(pthread_mutex_t * lock)
